/** Ikon Integration DynamoDB wrapper**/
import AWS from "aws-sdk";
import Utils from "./IKUtils";
//
export default class IKRequest {
  constructor(request, context) {
    this.requestEvent = request;
    this.context = context;
    console.debug("Request info:", JSON.stringify(request));
    console.debug("Request context:", JSON.stringify(context));
  }
  //query
  containsQueryParam(paramName) {
    const val = this.getQueryParam(paramName);
    return (Utils.isValidString(val) || Utils.isValidNumber(val));
  }
  getQueryParam(paramName) { return Utils.caseInsensitiveObjectForKey(this.requestEvent.queryStringParameters, paramName); }
  //header
  getHeader(headerName) { return Utils.caseInsensitiveObjectForKey(this.requestEvent.headers, headerName); }
  //context params
  getContextParam(cxtParam) { return Utils.caseInsensitiveObjectForKey(this.requestEvent.requestContext, cxtParam); }
  //path param
  containsPathParam(paramName) {
    const val = this.getPathParam(paramName);
    return (Utils.isValidString(val) || Utils.isValidNumber(val));
  }
  getPathParam(paramName) { return Utils.caseInsensitiveObjectForKey(this.requestEvent.pathParameters, paramName); }
  //body
  getBody() { 
    let b = null;
    try {
      b = this.requestEvent.body;
      if (typeof b === 'string' || b instanceof String) b = JSON.parse(b);
    } catch (e) {}
    return b;
  }
  getPath() { return this.requestEvent.path; }
  getMethod() { return this.requestEvent.httpMethod; }
  getPathParams() { return this.requestEvent.pathParameters; }

  //Shortcuts
  getAuthorizationHeader() { return this.getHeader('Authorization'); }
  getRequestID() {
    if (this.context.awsRequestId) return this.context.awsRequestId;
    return (this.requestEvent.requestContext ? this.requestEvent.requestContext.requestId : 'unknown');
  }
  getOriginIP() {
    const origin = this.getContextParam('identity').sourceIp;
    const hOrigin = this.getHeader('X-Forwarded-For');
    return (origin ? origin : (hOrigin ? hOrigin : 'unknow'));
  }
 }
