//Components
import Utils  from "../API/IKUtils";
import IKDBDDB from "../Database/DDB/IKDB";
import IKDBSQL from "../Database/SQL/IKDB";
import IKLogger from "../Logger/IKLogger";
import IKGlobals from '../IKGlobals';
import IKValidation from '../Validation/IKValidation';
import IKPublisher from '../Publisher/IKPublisher';
//
export default class IKProcess {
  constructor(config, interval) {
    this._config = config;
    this.interval = interval;
    this.logger = new IKLogger({/*COFIG S3/SQS HERE*/}, Utils.logLevel());
    this.publisher = new IKPublisher(config.publisher);
    this.validator = new IKValidation(config.validation);
    this.db = this._getDBDriver();
  }

  //Main interface
  async execute(executionFunc) {
    console.debug("Starting main process code");
    //Connect DB
    await this.db.connect();
    //program loop
    setInterval( async () => {
      await this._execute(executionFunc);
    }, this.interval);
  }
  //Executions
  async _execute(executionFunc) {
    let executionFailed = true; //failled til we say no!
    //safe execution handler
    try { 
      //start DB transaction
      await this.db.beginTransaction();
      //Execute
      await executionFunc(this);
      //Commit DB
      await this.db.commit();
      executionFailed = false;
    } catch (e) { /*EXECUTION FAIL*/
      console.error('Exception when executing main request code. Rolling back DB!');
      console.exception(e);
      //Rollback DB
      await this.db.rollback();
    } return executionFailed;
  }
  /* DB drivers support */
  _getDBDriver() {
    if (this._config.database) {
      if (this._config.database.type == IKGlobals.DBDrivers.SQL) return new IKDBSQL(this._config.database);
      else if (Object.keys(this._config.database).length > 0) return new IKDBDDB(this._config.database);
    } return null;
  }
}
