import IKDBBaseQuery from './IKDBBaseQuery';
import Utils from './../../API/IKUtils';
//
export default class IKDBQueryGet extends IKDBBaseQuery {
  constructor(optIndexName) {
    super(optIndexName);
    this.limit = -1;
    this.lastEvaluatedKey = null;
  }
  async run(dbManager, appendingItems) {
    const resp = await this.rawRun(dbManager, appendingItems);
    if (resp.Items) {
      let unmarshalled = Utils.decapsulateForDB(resp.Items);
      console.debug('Parsed result: ', unmarshalled);
      //Check for appending items from previous query
      if (appendingItems && appendingItems.length > 0) {
        unmarshalled = unmarshalled.concat(appendingItems);
      }
      //Check for continuation
      this.lastEvaluatedKey = resp.LastEvaluatedKey;
      if (resp.LastEvaluatedKey && !(this.limit > 0 && unmarshalled.length == this.limit)) {
        console.debug('Using continuation key:', resp.LastEvaluatedKey);
        this.exclusiveStartKey = resp.LastEvaluatedKey;
        return this.run(dbManager, unmarshalled);
      } return unmarshalled;
    } return null;
  }
  async rawRun(dbManager, appendingItems) {
    const query = {
      ...this._rawQuery(dbManager.tableName),
      ...(this.limit > 0 ? { Limit: this.limit - (appendingItems ? appendingItems.length : 0) } : {})
    };
    console.log('Querying: ', query);
    const resp = await dbManager.connection.query(query).promise();
    console.debug('Raw result: ', resp);
    return resp;
  }
}
