import IKTransaction from './BaseEvent/IKTransaction';
import IKResponse, { IKBadRequestResponse, IKSuccessResponse } from './API/IKResponse';
import IKGlobals from './IKGlobals';
import Utils from "./API/IKUtils";
//
export default class IKDynamoStream {
  constructor(event, context, config) {
    this.event = event;
    this.context = context;
    this.apiConfig = config;
  }
  async processEvent(execution) {
    const resp = await this._processRawEvent(execution);
    if (resp && resp.getCode() != 200) throw new Error(JSON.stringify(resp.body));
    else if (resp) return resp;
  }
  async _processRawEvent(execution) {
    if (this.event.Records && this.event.Records.length > 0) { //safe check for empty events?
      //init transaction for all records
      return await (new IKTransaction(this.event, this.context, this.apiConfig, true /*on queue*/)).execute(async (transaction) => {
        //for each available event
        for (let eventRecord of this.event.Records) {
          //Prepare parsed record
          const parsedRecord = {
            ...eventRecord,
            ...(eventRecord.dynamodb.Keys ? {Keys: Utils.decapsulateForDB(eventRecord.dynamodb.Keys)} : {}),
            ...(eventRecord.dynamodb.OldImage ? {OldImage: Utils.decapsulateForDB(eventRecord.dynamodb.OldImage)} : {}),
            ...(eventRecord.dynamodb.NewImage ? {NewImage: Utils.decapsulateForDB(eventRecord.dynamodb.NewImage)} : {}),
            //...(eventRecord.OldImage ? Utils.decapsulateForDB(eventRecord.OldImage) : {}),
          };
          //Call execution
          let resp = await execution(transaction, parsedRecord);
          if (resp.getCode() != 200 && resp.getCode() != 204) { //if execution failed, respond and mark as answered
            return resp;
          }
        }
        //not errored and loop ended - succeeded
        return IKSuccessResponse();
      });
    } else return IKBadRequestResponse(IKGlobals.ErrorResponseNoRecords, IKGlobals.ErrorCode_NoRecords); //no event to be processed?
  }
}
