import AWS from "aws-sdk";
//
import Utils from "./../API/IKUtils";
//
export default class IKPublisher {
  constructor(config) {
    if (config && config.region) {
      this.region = config.region;
      console.debug(`Using region: ${this.region}`);
    }
  }

  async publishOnTopic(messageObject, topic) {
    let resp = null;
    try {
      //Configure when sending!
      let configObj = { region: this.region };
      if (Utils.isOffline()) configObj = { ...configObj, accessKeyId: process.env.AWS_ACCESS_KEY_ID, secretAccessKey: process.env.AWS_SECRET_ACCESS_KEY };
      AWS.config.update(configObj);
      //Send to SNS
      const params = {
        Message: JSON.stringify(messageObject),
        TopicArn: topic
      };
      resp = await (new AWS.SNS({ apiVersion: '2010-03-31' })).publish(params).promise();
    } catch (e) {
      console.error(`Error while publishing into topic ${topic} with error: ${e}`);
    }
    console.debug('Publisher resp', resp);
    return resp;
  }
}
